@@ -23,6 +23,8 @@
 extern "C" {
 #endif
 
+#define LIBSMB2_SHARE_ENUM_V2 1
+
 struct smb2_iovec {
         uint8_t *buf;
         size_t len;
@@ -38,6 +40,37 @@ struct smb2_context;
 typedef void (*smb2_command_cb)(struct smb2_context *smb2, int status,
                                 void *command_data, void *cb_data);
 
+/*
+ * callback for getting error information when errors are set
+ * command_data depends on status.
+ */
+typedef void (*smb2_error_cb)(struct smb2_context *smb2,
+                                const char *error_string);
+
+/*
+ * callback for server accepting a new connection
+ */
+typedef int (*smb2_accepted_cb)(const int fd, void *cb_data);
+
+/*
+ * callback when a new connection is made to setup context
+ */
+typedef void (*smb2_client_connection)(struct smb2_context *smb2, void *cb_data);
+
+/*
+ * callback when a server notifies of an oplock or lease break
+ * the type of break is determined from the stuct_size in the request
+ * (notification) passed in.  the app can set the new oplock level
+ * or new lease state for the acknowledgement that will be sent back
+ */
+struct smb2_oplock_or_lease_break_reply;
+
+typedef void (*smb2_oplock_or_lease_break_cb)(struct smb2_context *smb2,
+           int status,
+           struct smb2_oplock_or_lease_break_reply *rep,
+           uint8_t *new_oplock_level,
+           uint32_t *new_lease_state);
+
 /* Stat structure */
 #define SMB2_TYPE_FILE      0x00000000
 #define SMB2_TYPE_DIRECTORY 0x00000001
@@ -53,8 +86,8 @@ struct smb2_stat_64 {
         uint64_t smb2_mtime_nsec;
         uint64_t smb2_ctime;
         uint64_t smb2_ctime_nsec;
-    uint64_t smb2_btime;
-    uint64_t smb2_btime_nsec;
+        uint64_t smb2_btime;
+        uint64_t smb2_btime_nsec;
 };
 
 struct smb2_statvfs {
@@ -76,8 +109,19 @@ struct smb2dirent {
         struct smb2_stat_64 st;
 };
 
-#ifdef _MSC_VER
+#if defined(_WINDOWS)
+#ifdef __USE_WINSOCK__
+#include <winsock.h>
+#else
+#include <ws2tcpip.h>
 #include <winsock2.h>
+#endif
+#elif defined(_XBOX)
+#include <xtl.h>
+#include <winsockx.h>
+#endif
+
+#if defined(_WINDOWS) || defined(_XBOX)
 typedef SOCKET t_socket;
 #else
 typedef int t_socket;
@@ -91,6 +135,15 @@ typedef int t_socket;
  */
 struct smb2_context *smb2_init_context(void);
 
+/*
+ * Close an SMB2 context
+ *
+ * closes socket if open, and clears keys but leave
+ * context allocated.  the context will be destroyed
+ * at a time later when it won't be in-use
+ */
+void smb2_close_context(struct smb2_context *smb2);
+
 /*
  * Destroy an smb2 context.
  *
@@ -102,6 +155,17 @@ struct smb2_context *smb2_init_context(void);
  */
 void smb2_destroy_context(struct smb2_context *smb2);
 
+/*
+ * Get the list of currently allocated contexts
+ */
+struct smb2_context *smb2_active_contexts(void);
+
+/*
+ * Determine of the context is currently active.  This lets
+ * code know if the context was destroyed in a callback for example
+ */
+int smb2_context_active(struct smb2_context *smb2);
+
 /*
  * EVENT SYSTEM INTEGRATION
  * ========================
@@ -159,8 +223,8 @@ smb2_get_fds(struct smb2_context *smb2, size_t *fd_count, int *timeout);
  */
 #define SMB2_ADD_FD 0
 #define SMB2_DEL_FD 1
-typedef void (*smb2_change_fd_cb)(struct smb2_context *smb2, int fd, int cmd);
-typedef void (*smb2_change_events_cb)(struct smb2_context *smb2, int fd,
+typedef void (*smb2_change_fd_cb)(struct smb2_context *smb2, t_socket fd, int cmd);
+typedef void (*smb2_change_events_cb)(struct smb2_context *smb2, t_socket fd,
                                       int events);
 void smb2_fd_event_callbacks(struct smb2_context *smb2,
                              smb2_change_fd_cb change_fd,
@@ -193,7 +257,7 @@ int smb2_service(struct smb2_context *smb2, int revents);
  *      used and must be freed by calling smb2_destroy_context().
  *
  */
-int smb2_service_fd(struct smb2_context *smb2, int fd, int revents);
+int smb2_service_fd(struct smb2_context *smb2, t_socket fd, int revents);
 
 /*
  * Set the timeout in seconds after which a command will be aborted with
@@ -205,6 +269,27 @@ int smb2_service_fd(struct smb2_context *smb2, int fd, int revents);
  */
 void smb2_set_timeout(struct smb2_context *smb2, int seconds);
 
+/*
+ * Set passthrough-enable.  Passthrough allows command packers
+ * and unpackers to keep the extra data on complex commands
+ * in its on-the-wire format without any interpretation. this
+ * is useful for proxy use cases where there might be no need
+ * to fully parse things like query-info replies or ioctl
+ * requests.  If passthrough is not set, any command that
+ * that is processed that can't interpret the data will fail
+ * Most client use case will not need this
+ *
+ * Default is 0: no-passthrough
+ */
+void smb2_set_passthrough(struct smb2_context *smb2,
+                      int passthrough);
+
+/*
+ * Get the current passthrough setting
+ */
+void smb2_get_passthrough(struct smb2_context *smb2,
+                      int *passthrough);
+
 /*
  * Set which version of SMB to negotiate.
  * Default is to let the server pick the version.
@@ -217,12 +302,40 @@ enum smb2_negotiate_version {
         SMB2_VERSION_0210 = 0x0210,
         SMB2_VERSION_0300 = 0x0300,
         SMB2_VERSION_0302 = 0x0302,
-        SMB2_VERSION_0311 = 0x0311
+        SMB2_VERSION_0311 = 0x0311,
 };
 
+#define SMB2_VERSION_WILDCARD 0x02FF
+
 void smb2_set_version(struct smb2_context *smb2,
                       enum smb2_negotiate_version version);
 
+/*
+ * Sets which version libsmb2 uses.
+*/
+#define LIBSMB2_MAJOR_VERSION 4
+#define LIBSMB2_MINOR_VERSION 0
+#define LIBSMB2_PATCH_VERSION 0
+
+struct smb2_libversion
+{
+     uint8_t major_version;
+     uint8_t minor_version;
+     uint8_t patch_version;
+};
+
+/*
+ * Gets the libsmb2 version being linked while used.
+ * This function will be available on 5.x
+ * @param struct smb2_libversion
+*/
+void smb2_get_libsmb2Version(struct smb2_libversion *smb2_ver);
+
+/*
+ * gets the (currently) negotiated dialect
+ */
+uint16_t smb2_get_dialect(struct smb2_context *smb2);
+
 /*
  * Set the security mode for the connection.
  * This is a combination of the flags SMB2_NEGOTIATE_SIGNING_ENABLED
@@ -245,6 +358,12 @@ void smb2_set_seal(struct smb2_context *smb2, int val);
  */
 void smb2_set_sign(struct smb2_context *smb2, int val);
 
+enum smb2_sec {
+        SMB2_SEC_UNDEFINED = 0,
+        SMB2_SEC_NTLMSSP,
+        SMB2_SEC_KRB5,
+};
+
 /*
  * Set authentication method.
  * SMB2_SEC_UNDEFINED (use KRB if available or NTLM if not)
@@ -258,22 +377,77 @@ void smb2_set_authentication(struct smb2_context *smb2, int val);
  * Default is to try to authenticate as the current user.
  */
 void smb2_set_user(struct smb2_context *smb2, const char *user);
+
+/*
+ * Get the username associated with a context.
+ * returns NULL if none
+ */
+const char *smb2_get_user(struct smb2_context *smb2);
+
 /*
  * Set the password that we will try to authenticate as.
  * This function is only needed when libsmb2 is built --without-libkrb5
  */
 void smb2_set_password(struct smb2_context *smb2, const char *password);
+
+/*
+ * Convert a win timestamp to a unix timeval
+ */
+void smb2_win_to_timeval(uint64_t smb2_time, struct smb2_timeval *tv);
+
+/*
+ * Convert unit timeval to a win timestamp
+ */
+uint64_t smb2_timeval_to_win(struct smb2_timeval *tv);
+
+/*
+ * set the context error string
+ */
+void smb2_set_error(struct smb2_context *smb2,
+                    const char *error_string, ...);
+
+/*
+ * Register an error callback, so any calls to smb2_set_error will call this
+ * function with the error string generated
+ */
+void smb2_register_error_callback(struct smb2_context *smb,
+                    smb2_error_cb error_cb);
+
+/*
+ * register for oplock or lease break callbacks
+ */
+void smb2_set_oplock_or_lease_break_callback(struct smb2_context *smb2,
+                    smb2_oplock_or_lease_break_cb cb);
+
+/*
+ * Set the smb2 context passworkd from a file (see NTLM_USER_FILE)
+ * depends on user/domain being already set in smb2 context
+ */
+void smb2_set_password_from_file(struct smb2_context *smb2);
+
 /*
  * Set the domain when authenticating.
  * This function is only needed when libsmb2 is built --without-libkrb5
  */
 void smb2_set_domain(struct smb2_context *smb2, const char *domain);
+
+/*
+ * Get the domain associated with a context.
+ * returns NULL if none
+ */
+const char *smb2_get_domain(struct smb2_context *smb2);
 /*
  * Set the workstation when authenticating.
  * This function is only needed when libsmb2 is built --without-libkrb5
  */
 void smb2_set_workstation(struct smb2_context *smb2, const char *workstation);
 
+/*
+ * Get the workstation associated with a context.
+ * returns NULL if none
+ */
+const char *smb2_get_workstation(struct smb2_context *smb2);
+
 /*
  * Sets the address to some user defined object. May be used to make
  * additional context data available in the async callbacks.
@@ -285,6 +459,18 @@ void smb2_set_opaque(struct smb2_context *smb2, void *opaque);
  */
 void *smb2_get_opaque(struct smb2_context *smb2);
 
+/*
+ * copy credential handle from one context to another (and set
+ * it NULL in source context)
+ * returns 0 if handle transferred,
+ * or -1 if not (no handle or no context, or not applicable)
+ */
+int smb2_delegate_credentials(struct smb2_context *in, struct smb2_context *out);
+
+/*
+ * Sets the client_guid for this context.
+ */
+void smb2_set_client_guid(struct smb2_context *smb2, const uint8_t guid[SMB2_GUID_SIZE]);
 
 /*
  * Returns the client_guid for this context.
@@ -368,11 +554,44 @@ int smb2_disconnect_share_async(struct smb2_context *smb2,
  */
 int smb2_disconnect_share(struct smb2_context *smb2);
 
+/*
+ * Select a tree id that was previously connected. Sets the tree_id
+ * in the context to be used for subsequent requests
+ *
+ * Returns:
+ * 0      : OK
+ * -errno : tree wasn't connected
+ */
+int smb2_select_tree_id(struct smb2_context *smb2, uint32_t tree_id);
+
+struct smb2_pdu;
+
+/*
+ * Get/Set the tree id of a pdu
+ *
+ * Returns:
+ * 0      : OK
+ * -errno : tree wasn't connected | no pdu | no context
+ */
+int smb2_get_tree_id_for_pdu(struct smb2_context *smb2, struct smb2_pdu *pdu, uint32_t *tree_id);
+int smb2_set_tree_id_for_pdu(struct smb2_context *smb2, struct smb2_pdu *pdu, uint32_t tree_id);
+
+/*
+ * Get session id
+ *
+ * Returns:
+ * 0      : OK
+ * -errno :
+ */
+int smb2_get_session_id(struct smb2_context *smb2, uint64_t *session_id);
+
 /*
  * This function returns a description of the last encountered error.
  */
 const char *smb2_get_error(struct smb2_context *smb2);
 
+int smb2_get_nterror(struct smb2_context *smb2);
+
 struct smb2_url {
         const char *domain;
         const char *user;
@@ -403,7 +622,6 @@ int nterror_to_errno(uint32_t status);
 struct smb2_url *smb2_parse_url(struct smb2_context *smb2, const char *url);
 void smb2_destroy_url(struct smb2_url *url);
 
-struct smb2_pdu;
 /*
  * The functions are used when creating compound low level commands.
  * The general pattern for compound chains is
@@ -424,6 +642,10 @@ void smb2_add_compound_pdu(struct smb2_context *smb2,
                            struct smb2_pdu *pdu, struct smb2_pdu *next_pdu);
 void smb2_free_pdu(struct smb2_context *smb2, struct smb2_pdu *pdu);
 void smb2_queue_pdu(struct smb2_context *smb2, struct smb2_pdu *pdu);
+void smb2_set_pdu_status(struct smb2_context *smb2, struct smb2_pdu *pdu, int status);
+void smb2_set_pdu_message_id(struct smb2_context *smb2, struct smb2_pdu *pdu, uint64_t message_id);
+uint64_t smb2_get_pdu_message_id(struct smb2_context *smb2, struct smb2_pdu *pdu);
+int smb2_pdu_is_compound(struct smb2_context *smb2);
 
 /*
  * OPENDIR
@@ -441,7 +663,7 @@ struct smb2dir;
  *      0 : Success.
  *          Command_data is struct smb2dir.
  *          This structure is freed using smb2_closedir().
- * -errno : An error occured.
+ * -errno : An error occurred.
  *          Command_data is NULL.
  */
 int smb2_opendir_async(struct smb2_context *smb2, const char *path,
@@ -521,9 +743,13 @@ struct smb2fh;
  *      0 : Success.
  *          Command_data is struct smb2fh.
  *          This structure is freed using smb2_close().
- * -errno : An error occured.
+ * -errno : An error occurred.
  *          Command_data is NULL.
  */
+int smb2_open_async_with_oplock_or_lease(struct smb2_context *smb2, const char *path, int flags,
+                    uint8_t oplock_level, uint32_t lease_state, smb2_lease_key lease_key,
+                    smb2_command_cb cb, void *cb_data);
+
 int smb2_open_async(struct smb2_context *smb2, const char *path, int flags,
                     smb2_command_cb cb, void *cb_data);
 
@@ -547,7 +773,7 @@ struct smb2fh *smb2_open(struct smb2_context *smb2, const char *path, int flags)
  *
  * When the callback is invoked, status indicates the result:
  *      0 : Success.
- * -errno : An error occured.
+ * -errno : An error occurred.
  *
  * Command_data is always NULL.
  */
@@ -572,7 +798,7 @@ int smb2_close(struct smb2_context *smb2, struct smb2fh *fh);
  *
  * When the callback is invoked, status indicates the result:
  *      0 : Success.
- * -errno : An error occured.
+ * -errno : An error occurred.
  *
  * Command_data is always NULL.
  */
@@ -620,7 +846,7 @@ struct smb2_write_cb_data {
  *
  * When the callback is invoked, status indicates the result:
  *    >=0 : Number of bytes read.
- * -errno : An error occured.
+ * -errno : An error occurred.
  *
  * Command_data is struct smb2_read_cb_data, which holds the arguments
  * that were given to smb2_pread_async.
@@ -653,7 +879,7 @@ int smb2_pread(struct smb2_context *smb2, struct smb2fh *fh,
  *
  * When the callback is invoked, status indicates the result:
  *    >=0 : Number of bytes written.
- * -errno : An error occured.
+ * -errno : An error occurred.
  *
  * Command_data is struct smb2_write_cb_data, which holds the arguments
  * that were given to smb2_pwrite_async.
@@ -684,7 +910,7 @@ int smb2_pwrite(struct smb2_context *smb2, struct smb2fh *fh,
  *
  * When the callback is invoked, status indicates the result:
  *    >=0 : Number of bytes read.
- * -errno : An error occured.
+ * -errno : An error occurred.
  *
  * Command_data is struct smb2_read_cb_data, which holds the arguments
  * that were given to smb2_read_async. offset denotes the offset in the file
@@ -714,7 +940,7 @@ int smb2_read(struct smb2_context *smb2, struct smb2fh *fh,
  *
  * When the callback is invoked, status indicates the result:
  *    >=0 : Number of bytes written.
- * -errno : An error occured.
+ * -errno : An error occurred.
  *
  * Command_data is struct smb2_write_cb_data, which holds the arguments
  * that were given to smb2_write_async. offset denotes the offset in the file
@@ -755,7 +981,7 @@ int64_t smb2_lseek(struct smb2_context *smb2, struct smb2fh *fh,
  *
  * When the callback is invoked, status indicates the result:
  *      0 : Success.
- * -errno : An error occured.
+ * -errno : An error occurred.
  *
  * Command_data is always NULL.
  */
@@ -780,7 +1006,7 @@ int smb2_unlink(struct smb2_context *smb2, const char *path);
  *
  * When the callback is invoked, status indicates the result:
  *      0 : Success.
- * -errno : An error occured.
+ * -errno : An error occurred.
  *
  * Command_data is always NULL.
  */
@@ -805,7 +1031,7 @@ int smb2_rmdir(struct smb2_context *smb2, const char *path);
  *
  * When the callback is invoked, status indicates the result:
  *      0 : Success.
- * -errno : An error occured.
+ * -errno : An error occurred.
  *
  * Command_data is always NULL.
  */
@@ -830,7 +1056,7 @@ int smb2_mkdir(struct smb2_context *smb2, const char *path);
  *
  * When the callback is invoked, status indicates the result:
  *      0 : Success. Command_data is struct smb2_statvfs
- * -errno : An error occured.
+ * -errno : An error occurred.
  */
 int smb2_statvfs_async(struct smb2_context *smb2, const char *path,
                        struct smb2_statvfs *statvfs,
@@ -854,7 +1080,7 @@ int smb2_statvfs(struct smb2_context *smb2, const char *path,
  *
  * When the callback is invoked, status indicates the result:
  *      0 : Success. Command_data is struct smb2_stat_64
- * -errno : An error occured.
+ * -errno : An error occurred.
  */
 int smb2_fstat_async(struct smb2_context *smb2, struct smb2fh *fh,
                      struct smb2_stat_64 *st,
@@ -875,7 +1101,7 @@ int smb2_fstat(struct smb2_context *smb2, struct smb2fh *fh,
  *
  * When the callback is invoked, status indicates the result:
  *      0 : Success. Command_data is struct smb2_stat_64
- * -errno : An error occured.
+ * -errno : An error occurred.
  */
 int smb2_stat_async(struct smb2_context *smb2, const char *path,
                     struct smb2_stat_64 *st,
@@ -896,7 +1122,7 @@ int smb2_stat(struct smb2_context *smb2, const char *path,
  *
  * When the callback is invoked, status indicates the result:
  *      0 : Success.
- * -errno : An error occured.
+ * -errno : An error occurred.
  */
 int smb2_rename_async(struct smb2_context *smb2, const char *oldpath,
                       const char *newpath, smb2_command_cb cb, void *cb_data);
@@ -917,7 +1143,7 @@ int smb2_rename(struct smb2_context *smb2, const char *oldpath,
  *
  * When the callback is invoked, status indicates the result:
  *      0 : Success.
- * -errno : An error occured.
+ * -errno : An error occurred.
  */
 int smb2_truncate_async(struct smb2_context *smb2, const char *path,
                         uint64_t length, smb2_command_cb cb, void *cb_data);
@@ -925,7 +1151,7 @@ int smb2_truncate_async(struct smb2_context *smb2, const char *path,
  * Sync truncate()
  * Function returns
  *      0 : Success
- * -errno : An error occured.
+ * -errno : An error occurred.
  */
 int smb2_truncate(struct smb2_context *smb2, const char *path,
                   uint64_t length);
@@ -940,7 +1166,7 @@ int smb2_truncate(struct smb2_context *smb2, const char *path,
  *
  * When the callback is invoked, status indicates the result:
  *      0 : Success.
- * -errno : An error occured.
+ * -errno : An error occurred.
  */
 int smb2_ftruncate_async(struct smb2_context *smb2, struct smb2fh *fh,
                          uint64_t length, smb2_command_cb cb, void *cb_data);
@@ -948,7 +1174,7 @@ int smb2_ftruncate_async(struct smb2_context *smb2, struct smb2fh *fh,
  * Sync ftruncate()
  * Function returns
  *      0 : Success
- * -errno : An error occured.
+ * -errno : An error occurred.
  */
 int smb2_ftruncate(struct smb2_context *smb2, struct smb2fh *fh,
                    uint64_t length);
@@ -967,7 +1193,7 @@ int smb2_ftruncate(struct smb2_context *smb2, struct smb2fh *fh,
  *
  * When the callback is invoked, status indicates the result:
  *      0 : Success. Command_data is the link content.
- * -errno : An error occured.
+ * -errno : An error occurred.
  */
 int smb2_readlink_async(struct smb2_context *smb2, const char *path,
                         smb2_command_cb cb, void *cb_data);
@@ -987,7 +1213,7 @@ int smb2_readlink(struct smb2_context *smb2, const char *path, char *buf, uint32
  *
  * When the callback is invoked, status indicates the result:
  *      0 : Success.
- * -errno : An error occured.
+ * -errno : An error occurred.
  */
 int smb2_echo_async(struct smb2_context *smb2,
                     smb2_command_cb cb, void *cb_data);
@@ -1001,6 +1227,141 @@ int smb2_echo_async(struct smb2_context *smb2,
  */
 int smb2_echo(struct smb2_context *smb2);
 
+void
+free_smb2_file_notify_change_information(struct smb2_context *smb2, struct smb2_file_notify_change_information *fnc);
+
+int smb2_notify_change_async(struct smb2_context *smb2, const char *path, uint16_t flags, uint32_t filter, int loop,
+                       smb2_command_cb cb, void *cb_data);
+
+int smb2_notify_change_filehandle_async(struct smb2_context *smb2, struct smb2fh *smb2_dir_fh, uint16_t flags, uint32_t filter, int loop,
+                       smb2_command_cb cb, void *cb_data);
+
+/*
+ * Sync notify_change()
+ *
+ */
+struct smb2_file_notify_change_information *smb2_notify_change(struct smb2_context *smb2, const char *path, uint16_t flags, uint32_t filter);
+
+/* Utilities that help by being public
+*/
+
+/* SMB's UTF-16 is always in Little Endian */
+struct smb2_utf16 {
+        int len;
+        uint16_t val[1];
+};
+
+/* Returns a string converted to UTF-16 format. Use free() to release
+ * the utf16 string.
+ */
+struct smb2_utf16 *smb2_utf8_to_utf16(const char *utf8);
+
+/* Returns a string converted to UTF8 format. Use free() to release
+ * the utf8 string.
+ */
+const char *smb2_utf16_to_utf8(const uint16_t *str, size_t len);
+
+/************* Server-side API **********************************************/
+struct smb2_server;
+
+/* pdu handlers in general take the request from the client, and return
+ * < 0  on error, and the library should create an error reply
+ * == 0 on OK, and the library should use the reply struct (if needed) to create a reply
+ * > 0  if the handler created and queued a reply itself
+ */
+struct smb2_server_request_handlers {
+        int (*destruction_event)(struct smb2_server *srvr, struct smb2_context *smb2);
+        int (*authorize_user)(struct smb2_server *srvr, struct smb2_context *smb2,
+                            const char *user,
+                            const char *domain,
+                            const char *workstation);
+        int (*session_established)(struct smb2_server *srvr, struct smb2_context *smb2);
+        int (*logoff_cmd)(struct smb2_server *srvr, struct smb2_context *smb2);
+        int (*tree_connect_cmd)(struct smb2_server *srvr, struct smb2_context *smb2,
+                            struct smb2_tree_connect_request *req,
+                            struct smb2_tree_connect_reply *rep);
+        int (*tree_disconnect_cmd)(struct smb2_server *srvr, struct smb2_context *smb2,
+                            const uint32_t tree_id);
+        int (*create_cmd)(struct smb2_server *srvr, struct smb2_context *smb2,
+                            struct smb2_create_request *req,
+                            struct smb2_create_reply *rep);
+        int (*close_cmd)(struct smb2_server *srvr, struct smb2_context *smb2,
+                            struct smb2_close_request *req,
+                            struct smb2_close_reply *rep);
+        int (*flush_cmd)(struct smb2_server *srvr, struct smb2_context *smb2,
+                            struct smb2_flush_request *req);
+        int (*read_cmd)(struct smb2_server *srvr, struct smb2_context *smb2,
+                            struct smb2_read_request *req,
+                            struct smb2_read_reply *rep);
+        int (*write_cmd)(struct smb2_server *srvr, struct smb2_context *smb2,
+                            struct smb2_write_request *req,
+                            struct smb2_write_reply *rep);
+        int (*oplock_break_cmd)(struct smb2_server *srvr, struct smb2_context *smb2,
+                            struct smb2_oplock_break_acknowledgement *req);
+        int (*lease_break_cmd)(struct smb2_server *srvr, struct smb2_context *smb2,
+                            struct smb2_lease_break_acknowledgement *req);
+        int (*lock_cmd)(struct smb2_server *srvr, struct smb2_context *smb2,
+                            struct smb2_lock_request *req);
+        int (*ioctl_cmd)(struct smb2_server *srvr, struct smb2_context *smb2,
+                            struct smb2_ioctl_request *req,
+                            struct smb2_ioctl_reply *rep);
+        int (*cancel_cmd)(struct smb2_server *srvr, struct smb2_context *smb2);
+        int (*echo_cmd)(struct smb2_server *srvr, struct smb2_context *smb2);
+        int (*query_directory_cmd)(struct smb2_server *srvr, struct smb2_context *smb2,
+                            struct smb2_query_directory_request *req,
+                            struct smb2_query_directory_reply *rep);
+        int (*change_notify_cmd)(struct smb2_server *srvr, struct smb2_context *smb2,
+                            struct smb2_change_notify_request *req,
+                            struct smb2_change_notify_reply *rep);
+        int (*query_info_cmd)(struct smb2_server *srvr, struct smb2_context *smb2,
+                            struct smb2_query_info_request *req,
+                            struct smb2_query_info_reply *rep);
+        int (*set_info_cmd)(struct smb2_server *srvr, struct smb2_context *smb2,
+                            struct smb2_set_info_request *req);
+        /*
+        int (*oplock_break cmd)(struct smb2_server *srvr, struct smb2_context *smb2,
+                            struct smb2_oplock_break_request *req);
+        */
+};
+
+struct smb2_server {
+        uint8_t guid[16];
+        char hostname[128];
+        char domain[128];
+        int fd;
+        uint16_t port;
+        uint64_t session_counter;
+        struct smb2_server_request_handlers *handlers;
+        uint32_t max_transact_size;
+        uint32_t max_read_size;
+        uint32_t max_write_size;
+        int signing_enabled;
+        int allow_anonymous;
+        /* this can be set non-0 to delegate client authentication to
+         * another client and allow any authentication to this server */
+        int proxy_authentication;
+        /* saved from negotiate to be used in validate negotiate info */
+        uint32_t capabilities;
+        uint32_t security_mode;
+        /* for kerberos, credential context */
+        char keytab_path[256];
+        char error[128];
+        void *auth_data;
+};
+
+int smb2_bind_and_listen(const uint16_t port, const int max_connections, int *out_fd);
+int smb2_accept_connection_async(const int fd, const int to_msecs, smb2_accepted_cb cb, void *cb_data);
+int smb2_serve_port_async(const int fd, const int to_msecs, struct smb2_context **out_smb2);
+
+/*
+ * Sync serve port()
+ *
+ * Returns
+ *  0     : The server is complete by exiting its loop normally (shouldnt happen)
+ * -errno : There was an error causing server loop to exit
+ */
+int smb2_serve_port(struct smb2_server *server, const int max_connections, smb2_client_connection cb, void *cb_data);
+
 /*
  * Some symbols have moved over to a different header file to allow better
  * separation between dcerpc and smb2, so we need to include this header
